@@ -1,20 +1,20 @@
 /* Directory hashing for GNU Make.
 Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 Free Software
-Foundation, Inc.
+1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
+2010 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
-Foundation; either version 2, or (at your option) any later version.
+Foundation; either version 3 of the License, or (at your option) any later
+version.
 
 GNU Make is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
-GNU Make; see the file COPYING.  If not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.  */
+this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "make.h"
 #include "hash.h"
@@ -23,7 +23,8 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.  */
 # include <dirent.h>
 # define NAMLEN(dirent) strlen((dirent)->d_name)
 # ifdef VMS
-extern char *vmsify PARAMS ((char *name, int type));
+/* its prototype is in vmsdir.h, which is not needed for HAVE_DIRENT_H */
+const char *vmsify (const char *name, int type);
 # endif
 #else
 # define dirent direct
@@ -67,8 +68,8 @@ extern char *vmsify PARAMS ((char *name, int type));
 #define _USE_LFN 0
 #endif
 
-static char *
-dosify (char *filename)
+static const char *
+dosify (const char *filename)
 {
   static char dos_filename[14];
   char *df;
@@ -117,24 +118,21 @@ dosify (char *filename)
 #endif
 
 #ifdef HAVE_CASE_INSENSITIVE_FS
-static char *
-downcase (char *filename)
+static const char *
+downcase (const char *filename)
 {
   static PATH_VAR (new_filename);
   char *df;
-  int i;
 
   if (filename == 0)
     return 0;
 
   df = new_filename;
-
-  /* First, transform the name part.  */
-  for (i = 0; *filename != '\0'; ++i)
-  {
-    *df++ = tolower ((unsigned char)*filename);
-    ++filename;
-  }
+  while (*filename != '\0')
+    {
+      *df++ = tolower ((unsigned char)*filename);
+      ++filename;
+    }
 
   *df = 0;
 
@@ -145,7 +143,7 @@ downcase (char *filename)
 #ifdef VMS
 
 static int
-vms_hash (char *name)
+vms_hash (const char *name)
 {
   int h = 0;
   int g;
@@ -171,7 +169,7 @@ vms_hash (char *name)
 
 /* fake stat entry for a directory */
 static int
-vmsstat_dir (char *name, struct stat *st)
+vmsstat_dir (const char *name, struct stat *st)
 {
   char *s;
   int h;
@@ -184,6 +182,7 @@ vmsstat_dir (char *name, struct stat *st)
   s = strchr (name, ':');	/* find device */
   if (s)
     {
+      /* to keep the compiler happy we said "const char *name", now we cheat */
       *s++ = 0;
       st->st_dev = (char *)vms_hash (name);
       h = vms_hash (s);
@@ -192,8 +191,7 @@ vmsstat_dir (char *name, struct stat *st)
   else
     {
       st->st_dev = 0;
-      s = name;
-      h = vms_hash (s);
+      h = vms_hash (name);
     }
 
   st->st_ino[0] = h & 0xff;
@@ -214,27 +212,24 @@ struct directory_contents
   {
     dev_t dev;			/* Device and inode numbers of this dir.  */
 #ifdef WINDOWS32
-    /*
-     * Inode means nothing on WINDOWS32. Even file key information is
-     * unreliable because it is random per file open and undefined
-     * for remote filesystems. The most unique attribute I can
-     * come up with is the fully qualified name of the directory. Beware
-     * though, this is also unreliable. I'm open to suggestion on a better
-     * way to emulate inode.
-     */
+    /* Inode means nothing on WINDOWS32. Even file key information is
+     * unreliable because it is random per file open and undefined for remote
+     * filesystems. The most unique attribute I can come up with is the fully
+     * qualified name of the directory. Beware though, this is also
+     * unreliable. I'm open to suggestion on a better way to emulate inode.  */
     char *path_key;
     int   ctime;
     int   mtime;        /* controls check for stale directory cache */
     int   fs_flags;     /* FS_FAT, FS_NTFS, ... */
-#define FS_FAT      0x1
-#define FS_NTFS     0x2
-#define FS_UNKNOWN  0x4
+# define FS_FAT      0x1
+# define FS_NTFS     0x2
+# define FS_UNKNOWN  0x4
 #else
-#ifdef VMS
+# ifdef VMS
     ino_t ino[3];
-#else
+# else
     ino_t ino;
-#endif
+# endif
 #endif /* WINDOWS32 */
     struct hash_table dirfiles;	/* Files in this directory.  */
     DIR *dirstream;		/* Stream reading this directory.  */
@@ -243,7 +238,7 @@ struct directory_contents
 static unsigned long
 directory_contents_hash_1 (const void *key_0)
 {
-  struct directory_contents const *key = (struct directory_contents const *) key_0;
+  const struct directory_contents *key = key_0;
   unsigned long hash;
 
 #ifdef WINDOWS32
@@ -266,7 +261,7 @@ directory_contents_hash_1 (const void *key_0)
 static unsigned long
 directory_contents_hash_2 (const void *key_0)
 {
-  struct directory_contents const *key = (struct directory_contents const *) key_0;
+  const struct directory_contents *key = key_0;
   unsigned long hash;
 
 #ifdef WINDOWS32
@@ -301,8 +296,8 @@ directory_contents_hash_2 (const void *key_0)
 static int
 directory_contents_hash_cmp (const void *xv, const void *yv)
 {
-  struct directory_contents const *x = (struct directory_contents const *) xv;
-  struct directory_contents const *y = (struct directory_contents const *) yv;
+  const struct directory_contents *x = xv;
+  const struct directory_contents *y = yv;
   int result;
 
 #ifdef WINDOWS32
@@ -338,7 +333,7 @@ static struct hash_table directory_contents;
 
 struct directory
   {
-    char *name;			/* Name of the directory.  */
+    const char *name;			/* Name of the directory.  */
 
     /* The directory's contents.  This data may be shared by several
        entries in the hash table, which refer to the same directory
@@ -349,20 +344,20 @@ struct directory
 static unsigned long
 directory_hash_1 (const void *key)
 {
-  return_ISTRING_HASH_1 (((struct directory const *) key)->name);
+  return_ISTRING_HASH_1 (((const struct directory *) key)->name);
 }
 
 static unsigned long
 directory_hash_2 (const void *key)
 {
-  return_ISTRING_HASH_2 (((struct directory const *) key)->name);
+  return_ISTRING_HASH_2 (((const struct directory *) key)->name);
 }
 
 static int
 directory_hash_cmp (const void *x, const void *y)
 {
-  return_ISTRING_COMPARE (((struct directory const *) x)->name,
-			  ((struct directory const *) y)->name);
+  return_ISTRING_COMPARE (((const struct directory *) x)->name,
+			  ((const struct directory *) y)->name);
 }
 
 /* Table of directories hashed by name.  */
@@ -379,7 +374,7 @@ static unsigned int open_directories = 0;
 
 struct dirfile
   {
-    char *name;			/* Name of the file.  */
+    const char *name;		/* Name of the file.  */
     short length;
     short impossible;		/* This file is impossible.  */
   };
@@ -399,8 +394,8 @@ dirfile_hash_2 (const void *key)
 static int
 dirfile_hash_cmp (const void *xv, const void *yv)
 {
-  struct dirfile const *x = ((struct dirfile const *) xv);
-  struct dirfile const *y = ((struct dirfile const *) yv);
+  const struct dirfile *x = xv;
+  const struct dirfile *y = yv;
   int result = x->length - y->length;
   if (result)
     return result;
@@ -411,17 +406,18 @@ dirfile_hash_cmp (const void *xv, const void *yv)
 #define DIRFILE_BUCKETS 107
 #endif
 
-static int dir_contents_file_exists_p PARAMS ((struct directory_contents *dir, char *filename));
-static struct directory *find_directory PARAMS ((char *name));
+static int dir_contents_file_exists_p (struct directory_contents *dir,
+                                       const char *filename);
+static struct directory *find_directory (const char *name);
 
 /* Find the directory named NAME and return its `struct directory'.  */
 
 static struct directory *
-find_directory (char *name)
+find_directory (const char *name)
 {
-  register char *p;
-  register struct directory *dir;
-  register struct directory **dir_slot;
+  const char *p;
+  struct directory *dir;
+  struct directory **dir_slot;
   struct directory dir_key;
   int r;
 #ifdef WINDOWS32
@@ -450,32 +446,39 @@ find_directory (char *name)
       /* The directory was not found.  Create a new entry for it.  */
 
       p = name + strlen (name);
-      dir = (struct directory *) xmalloc (sizeof (struct directory));
-      dir->name = savestring (name, p - name);
+      dir = xmalloc (sizeof (struct directory));
+#if defined(HAVE_CASE_INSENSITIVE_FS) && defined(VMS)
+      dir->name = strcache_add_len (downcase(name), p - name);
+#else
+      dir->name = strcache_add_len (name, p - name);
+#endif
       hash_insert_at (&directories, dir, dir_slot);
       /* The directory is not in the name hash table.
 	 Find its device and inode numbers, and look it up by them.  */
 
-#ifdef WINDOWS32
-      /* Remove any trailing '\'.  Windows32 stat fails even on valid
-         directories if they end in '\'. */
-      if (p[-1] == '\\')
-        p[-1] = '\0';
-#endif
-
 #ifdef VMS
       r = vmsstat_dir (name, &st);
+#elif defined(WINDOWS32)
+      {
+        char tem[MAXPATHLEN], *tstart, *tend;
+
+        /* Remove any trailing slashes.  Windows32 stat fails even on
+           valid directories if they end in a slash. */
+        memcpy (tem, name, p - name + 1);
+        tstart = tem;
+        if (tstart[1] == ':')
+          tstart += 2;
+        for (tend = tem + (p - name - 1);
+             tend > tstart && (*tend == '/' || *tend == '\\');
+             tend--)
+          *tend = '\0';
+
+        r = stat (tem, &st);
+      }
 #else
       EINTRLOOP (r, stat (name, &st));
 #endif
 
-#ifdef WINDOWS32
-      /* Put back the trailing '\'.  If we don't, we're permanently
-         truncating the value!  */
-      if (p[-1] == '\0')
-        p[-1] = '\\';
-#endif
-
       if (r < 0)
         {
 	/* Couldn't stat the directory.  Mark this by
@@ -549,8 +552,8 @@ find_directory (char *name)
 	      hash_insert_at (&directory_contents, dc, dc_slot);
 	      ENULLLOOP (dc->dirstream, opendir (name));
 	      if (dc->dirstream == 0)
-                /* Couldn't open the directory.  Mark this by
-                   setting the `files' member to a nil pointer.  */
+                /* Couldn't open the directory.  Mark this by setting the
+                   `files' member to a nil pointer.  */
                 dc->dirfiles.ht_vec = 0;
 	      else
 		{
@@ -561,7 +564,7 @@ find_directory (char *name)
 		  if (open_directories == MAX_OPEN_DIRECTORIES)
 		    /* We have too many directories open already.
 		       Read the entire directory and then close it.  */
-		    (void) dir_contents_file_exists_p (dc, (char *) 0);
+		    dir_contents_file_exists_p (dc, 0);
 		}
 	    }
 
@@ -577,7 +580,8 @@ find_directory (char *name)
    FILENAME must contain no slashes.  */
 
 static int
-dir_contents_file_exists_p (struct directory_contents *dir, char *filename)
+dir_contents_file_exists_p (struct directory_contents *dir,
+                            const char *filename)
 {
   unsigned int hash;
   struct dirfile *df;
@@ -588,10 +592,9 @@ dir_contents_file_exists_p (struct directory_contents *dir, char *filename)
 #endif
 
   if (dir == 0 || dir->dirfiles.ht_vec == 0)
-    {
     /* The directory could not be stat'd or opened.  */
-      return 0;
-    }
+    return 0;
+
 #ifdef __MSDOS__
   filename = dosify (filename);
 #endif
@@ -621,11 +624,9 @@ dir_contents_file_exists_p (struct directory_contents *dir, char *filename)
 	}
       dirfile_key.name = filename;
       dirfile_key.length = strlen (filename);
-      df = (struct dirfile *) hash_find_item (&dir->dirfiles, &dirfile_key);
+      df = hash_find_item (&dir->dirfiles, &dirfile_key);
       if (df)
-	{
-	  return !df->impossible;
-	}
+        return !df->impossible;
     }
 
   /* The file was not found in the hashed list.
@@ -646,7 +647,7 @@ dir_contents_file_exists_p (struct directory_contents *dir, char *filename)
 	      dir->mtime = time ((time_t *) 0);
 	      rehash = 1;
 	    }
-	  else if (stat(dir->path_key, &st) == 0 && st.st_mtime > dir->mtime)
+	  else if (stat (dir->path_key, &st) == 0 && st.st_mtime > dir->mtime)
 	    {
 	      /* reset date stamp to show most recent re-process.  */
 	      dir->mtime = st.st_mtime;
@@ -658,7 +659,7 @@ dir_contents_file_exists_p (struct directory_contents *dir, char *filename)
 	    return 0;
 
           /* make sure directory can still be opened; if not return.  */
-          dir->dirstream = opendir(dir->path_key);
+          dir->dirstream = opendir (dir->path_key);
           if (!dir->dirstream)
             return 0;
 	}
@@ -677,7 +678,11 @@ dir_contents_file_exists_p (struct directory_contents *dir, char *filename)
 
       ENULLLOOP (d, readdir (dir->dirstream));
       if (d == 0)
-        break;
+        {
+          if (errno)
+            fatal (NILF, "INTERNAL: readdir: %s\n", strerror (errno));
+          break;
+        }
 
 #if defined(VMS) && defined(HAVE_DIRENT_H)
       /* In VMS we get file versions too, which have to be stripped off */
@@ -702,17 +707,19 @@ dir_contents_file_exists_p (struct directory_contents *dir, char *filename)
       if (! rehash || HASH_VACANT (*dirfile_slot))
 #endif
 	{
-	  df = (struct dirfile *) xmalloc (sizeof (struct dirfile));
-	  df->name = savestring (d->d_name, len);
+	  df = xmalloc (sizeof (struct dirfile));
+#if defined(HAVE_CASE_INSENSITIVE_FS) && defined(VMS)
+          df->name = strcache_add_len (downcase(d->d_name), len);
+#else
+	  df->name = strcache_add_len (d->d_name, len);
+#endif
 	  df->length = len;
 	  df->impossible = 0;
 	  hash_insert_at (&dir->dirfiles, df, dirfile_slot);
 	}
       /* Check if the name matches the one we're searching for.  */
-      if (filename != 0 && strieq (d->d_name, filename))
-	{
-	  return 1;
-	}
+      if (filename != 0 && patheq (d->d_name, filename))
+        return 1;
     }
 
   /* If the directory has been completely read in,
@@ -731,7 +738,7 @@ dir_contents_file_exists_p (struct directory_contents *dir, char *filename)
    FILENAME must contain no slashes.  */
 
 int
-dir_file_exists_p (char *dirname, char *filename)
+dir_file_exists_p (const char *dirname, const char *filename)
 {
   return dir_contents_file_exists_p (find_directory (dirname)->contents,
 				     filename);
@@ -740,11 +747,11 @@ dir_file_exists_p (char *dirname, char *filename)
 /* Return 1 if the file named NAME exists.  */
 
 int
-file_exists_p (char *name)
+file_exists_p (const char *name)
 {
-  char *dirend;
-  char *dirname;
-  char *slash;
+  const char *dirend;
+  const char *dirname;
+  const char *slash;
 
 #ifndef	NO_ARCHIVES
   if (ar_name (name))
@@ -755,14 +762,14 @@ file_exists_p (char *name)
   dirend = strrchr (name, ']');
   if (dirend == 0)
     dirend = strrchr (name, ':');
-  if (dirend == (char *)0)
+  if (dirend == 0)
     return dir_file_exists_p ("[]", name);
 #else /* !VMS */
   dirend = strrchr (name, '/');
 #ifdef HAVE_DOS_PATHS
   /* Forward and backslashes might be mixed.  We need the rightmost one.  */
   {
-    char *bslash = strrchr(name, '\\');
+    const char *bslash = strrchr(name, '\\');
     if (!dirend || bslash > dirend)
       dirend = bslash;
     /* The case of "d:file".  */
@@ -783,15 +790,17 @@ file_exists_p (char *name)
     dirname = "/";
   else
     {
+      char *p;
 #ifdef HAVE_DOS_PATHS
   /* d:/ and d: are *very* different...  */
       if (dirend < name + 3 && name[1] == ':' &&
 	  (*dirend == '/' || *dirend == '\\' || *dirend == ':'))
 	dirend++;
 #endif
-      dirname = (char *) alloca (dirend - name + 1);
-      bcopy (name, dirname, dirend - name);
-      dirname[dirend - name] = '\0';
+      p = alloca (dirend - name + 1);
+      memcpy (p, name, dirend - name);
+      p[dirend - name] = '\0';
+      dirname = p;
     }
   return dir_file_exists_p (dirname, slash + 1);
 }
@@ -801,12 +810,12 @@ file_exists_p (char *name)
    as an intermediate file, and it has failed.  */
 
 void
-file_impossible (char *filename)
+file_impossible (const char *filename)
 {
-  char *dirend;
-  register char *p = filename;
-  register struct directory *dir;
-  register struct dirfile *new;
+  const char *dirend;
+  const char *p = filename;
+  struct directory *dir;
+  struct dirfile *new;
 
 #ifdef VMS
   dirend = strrchr (p, ']');
@@ -820,7 +829,7 @@ file_impossible (char *filename)
 # ifdef HAVE_DOS_PATHS
   /* Forward and backslashes might be mixed.  We need the rightmost one.  */
   {
-    char *bslash = strrchr(p, '\\');
+    const char *bslash = strrchr(p, '\\');
     if (!dirend || bslash > dirend)
       dirend = bslash;
     /* The case of "d:file".  */
@@ -837,34 +846,32 @@ file_impossible (char *filename)
 #endif /* VMS */
   else
     {
-      char *dirname;
-      char *slash = dirend;
+      const char *dirname;
+      const char *slash = dirend;
       if (dirend == p)
 	dirname = "/";
       else
 	{
+          char *cp;
 #ifdef HAVE_DOS_PATHS
 	  /* d:/ and d: are *very* different...  */
 	  if (dirend < p + 3 && p[1] == ':' &&
 	      (*dirend == '/' || *dirend == '\\' || *dirend == ':'))
 	    dirend++;
 #endif
-	  dirname = (char *) alloca (dirend - p + 1);
-	  bcopy (p, dirname, dirend - p);
-	  dirname[dirend - p] = '\0';
+	  cp = alloca (dirend - p + 1);
+	  memcpy (cp, p, dirend - p);
+	  cp[dirend - p] = '\0';
+          dirname = cp;
 	}
       dir = find_directory (dirname);
       filename = p = slash + 1;
     }
 
   if (dir->contents == 0)
-    {
-      /* The directory could not be stat'd.  We allocate a contents
-	 structure for it, but leave it out of the contents hash table.  */
-      dir->contents = (struct directory_contents *)
-	xmalloc (sizeof (struct directory_contents));
-      bzero ((char *) dir->contents, sizeof (struct directory_contents));
-    }
+    /* The directory could not be stat'd.  We allocate a contents
+       structure for it, but leave it out of the contents hash table.  */
+    dir->contents = xcalloc (sizeof (struct directory_contents));
 
   if (dir->contents->dirfiles.ht_vec == 0)
     {
@@ -874,9 +881,13 @@ file_impossible (char *filename)
 
   /* Make a new entry and put it in the table.  */
 
-  new = (struct dirfile *) xmalloc (sizeof (struct dirfile));
-  new->name = xstrdup (filename);
+  new = xmalloc (sizeof (struct dirfile));
   new->length = strlen (filename);
+#if defined(HAVE_CASE_INSENSITIVE_FS) && defined(VMS)
+  new->name = strcache_add_len (downcase(filename), new->length);
+#else
+  new->name = strcache_add_len (filename, new->length);
+#endif
   new->impossible = 1;
   hash_insert (&dir->contents->dirfiles, new);
 }
@@ -884,12 +895,12 @@ file_impossible (char *filename)
 /* Return nonzero if FILENAME has been marked impossible.  */
 
 int
-file_impossible_p (char *filename)
+file_impossible_p (const char *filename)
 {
-  char *dirend;
-  register char *p = filename;
-  register struct directory_contents *dir;
-  register struct dirfile *dirfile;
+  const char *dirend;
+  const char *p = filename;
+  struct directory_contents *dir;
+  struct dirfile *dirfile;
   struct dirfile dirfile_key;
 
 #ifdef VMS
@@ -901,7 +912,7 @@ file_impossible_p (char *filename)
 #ifdef HAVE_DOS_PATHS
   /* Forward and backslashes might be mixed.  We need the rightmost one.  */
   {
-    char *bslash = strrchr(filename, '\\');
+    const char *bslash = strrchr(filename, '\\');
     if (!dirend || bslash > dirend)
       dirend = bslash;
     /* The case of "d:file".  */
@@ -918,21 +929,23 @@ file_impossible_p (char *filename)
 #endif /* VMS */
   else
     {
-      char *dirname;
-      char *slash = dirend;
+      const char *dirname;
+      const char *slash = dirend;
       if (dirend == filename)
 	dirname = "/";
       else
 	{
+          char *cp;
 #ifdef HAVE_DOS_PATHS
 	  /* d:/ and d: are *very* different...  */
 	  if (dirend < filename + 3 && filename[1] == ':' &&
 	      (*dirend == '/' || *dirend == '\\' || *dirend == ':'))
 	    dirend++;
 #endif
-	  dirname = (char *) alloca (dirend - filename + 1);
-	  bcopy (p, dirname, dirend - p);
-	  dirname[dirend - p] = '\0';
+	  cp = alloca (dirend - filename + 1);
+	  memcpy (cp, p, dirend - p);
+	  cp[dirend - p] = '\0';
+          dirname = cp;
 	}
       dir = find_directory (dirname)->contents;
       p = filename = slash + 1;
@@ -954,7 +967,7 @@ file_impossible_p (char *filename)
 
   dirfile_key.name = filename;
   dirfile_key.length = strlen (filename);
-  dirfile = (struct dirfile *) hash_find_item (&dir->dirfiles, &dirfile_key);
+  dirfile = hash_find_item (&dir->dirfiles, &dirfile_key);
   if (dirfile)
     return dirfile->impossible;
 
@@ -964,8 +977,8 @@ file_impossible_p (char *filename)
 /* Return the already allocated name in the
    directory hash table that matches DIR.  */
 
-char *
-dir_name (char *dir)
+const char *
+dir_name (const char *dir)
 {
   return find_directory (dir)->name;
 }
@@ -975,10 +988,10 @@ dir_name (char *dir)
 void
 print_dir_data_base (void)
 {
-  register unsigned int files;
-  register unsigned int impossible;
-  register struct directory **dir_slot;
-  register struct directory **dir_end;
+  unsigned int files;
+  unsigned int impossible;
+  struct directory **dir_slot;
+  struct directory **dir_end;
 
   puts (_("\n# Directories\n"));
 
@@ -988,7 +1001,7 @@ print_dir_data_base (void)
   dir_end = dir_slot + directories.ht_size;
   for ( ; dir_slot < dir_end; dir_slot++)
     {
-      register struct directory *dir = *dir_slot;
+      struct directory *dir = *dir_slot;
       if (! HASH_VACANT (dir))
 	{
 	  if (dir->contents == 0)
@@ -1013,16 +1026,16 @@ print_dir_data_base (void)
 	    }
 	  else
 	    {
-	      register unsigned int f = 0;
-	      register unsigned int im = 0;
-	      register struct dirfile **files_slot;
-	      register struct dirfile **files_end;
+	      unsigned int f = 0;
+	      unsigned int im = 0;
+	      struct dirfile **files_slot;
+	      struct dirfile **files_end;
 
 	      files_slot = (struct dirfile **) dir->contents->dirfiles.ht_vec;
 	      files_end = files_slot + dir->contents->dirfiles.ht_size;
 	      for ( ; files_slot < files_end; files_slot++)
 		{
-		  register struct dirfile *df = *files_slot;
+		  struct dirfile *df = *files_slot;
 		  if (! HASH_VACANT (df))
 		    {
 		      if (df->impossible)
@@ -1092,14 +1105,14 @@ struct dirstream
   };
 
 /* Forward declarations.  */
-static __ptr_t open_dirstream PARAMS ((const char *));
-static struct dirent *read_dirstream PARAMS ((__ptr_t));
+static __ptr_t open_dirstream (const char *);
+static struct dirent *read_dirstream (__ptr_t);
 
 static __ptr_t
 open_dirstream (const char *directory)
 {
   struct dirstream *new;
-  struct directory *dir = find_directory ((char *)directory);
+  struct directory *dir = find_directory (directory);
 
   if (dir->contents == 0 || dir->contents->dirfiles.ht_vec == 0)
     /* DIR->contents is nil if the directory could not be stat'd.
@@ -1109,9 +1122,9 @@ open_dirstream (const char *directory)
   /* Read all the contents of the directory now.  There is no benefit
      in being lazy, since glob will want to see every file anyway.  */
 
-  (void) dir_contents_file_exists_p (dir->contents, (char *) 0);
+  dir_contents_file_exists_p (dir->contents, 0);
 
-  new = (struct dirstream *) xmalloc (sizeof (struct dirstream));
+  new = xmalloc (sizeof (struct dirstream));
   new->contents = dir->contents;
   new->dirfile_slot = (struct dirfile **) new->contents->dirfiles.ht_vec;
 
@@ -1121,29 +1134,28 @@ open_dirstream (const char *directory)
 static struct dirent *
 read_dirstream (__ptr_t stream)
 {
-  struct dirstream *const ds = (struct dirstream *) stream;
-  struct directory_contents *dc = ds->contents;
-  struct dirfile **dirfile_end = (struct dirfile **) dc->dirfiles.ht_vec + dc->dirfiles.ht_size;
   static char *buf;
   static unsigned int bufsz;
 
+  struct dirstream *const ds = (struct dirstream *) stream;
+  struct directory_contents *dc = ds->contents;
+  struct dirfile **dirfile_end = (struct dirfile **) dc->dirfiles.ht_vec + dc->dirfiles.ht_size;
+
   while (ds->dirfile_slot < dirfile_end)
     {
-      register struct dirfile *df = *ds->dirfile_slot++;
+      struct dirfile *df = *ds->dirfile_slot++;
       if (! HASH_VACANT (df) && !df->impossible)
 	{
-	  /* The glob interface wants a `struct dirent',
-	     so mock one up.  */
+	  /* The glob interface wants a `struct dirent', so mock one up.  */
 	  struct dirent *d;
 	  unsigned int len = df->length + 1;
-	  if (sizeof *d - sizeof d->d_name + len > bufsz)
+          unsigned int sz = sizeof (*d) - sizeof (d->d_name) + len;
+	  if (sz > bufsz)
 	    {
-	      if (buf != 0)
-		free (buf);
 	      bufsz *= 2;
-	      if (sizeof *d - sizeof d->d_name + len > bufsz)
-		bufsz = sizeof *d - sizeof d->d_name + len;
-	      buf = xmalloc (bufsz);
+	      if (sz > bufsz)
+		bufsz = sz;
+	      buf = xrealloc (buf, bufsz);
 	    }
 	  d = (struct dirent *) buf;
 #ifdef __MINGW32__
@@ -1180,7 +1192,7 @@ ansi_free (void *p)
  */
 #ifndef stat
 # ifndef VMS
-extern int stat PARAMS ((const char *path, struct stat *sbuf));
+int stat (const char *path, struct stat *sbuf);
 # endif
 # define local_stat stat
 #else
@@ -1197,7 +1209,6 @@ local_stat (const char *path, struct stat *buf)
 void
 dir_setup_glob (glob_t *gl)
 {
-  /* Bogus sunos4 compiler complains (!) about & before functions.  */
   gl->gl_opendir = open_dirstream;
   gl->gl_readdir = read_dirstream;
   gl->gl_closedir = ansi_free;
@@ -1212,5 +1223,6 @@ hash_init_directories (void)
   hash_init (&directories, DIRECTORY_BUCKETS,
 	     directory_hash_1, directory_hash_2, directory_hash_cmp);
   hash_init (&directory_contents, DIRECTORY_BUCKETS,
-	     directory_contents_hash_1, directory_contents_hash_2, directory_contents_hash_cmp);
+	     directory_contents_hash_1, directory_contents_hash_2,
+             directory_contents_hash_cmp);
 }
